/*++

Copyright (c) 1992-1996  Microsoft Corporation

Module Name:

    pslog.h

Abstract:

    Constant definitions for the Intermediate sample error code log values.

Author:

    Jim MAteer 4/1/97

Revision History:

--*/

#ifndef _IMLOG_
#define _IMLOG_

//
//  Status values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-------------------------+-------------------------------+
//  |Sev|C|       Facility          |               Code            |
//  +---+-+-------------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//

//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      R - is a reserved bit
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//
// Define the facility codes
//
#define FACILITY_RPC_STUBS               0x3
#define FACILITY_RPC_RUNTIME             0x2
#define FACILITY_IMSAMP_ERROR_CODE       0x8
#define FACILITY_IO_ERROR_CODE           0x4


//
// Define the severity codes
//
#define STATUS_SEVERITY_WARNING          0x2
#define STATUS_SEVERITY_SUCCESS          0x0
#define STATUS_SEVERITY_INFORMATIONAL    0x1
#define STATUS_SEVERITY_ERROR            0x3


//
// MessageId: IM_EVENT_BAD_IM_REGISTRATION
//
// MessageText:
//
//  %2 cannot register with %3
//
#define IM_EVENT_BAD_IM_REGISTRATION    ((NTSTATUS)0xC0080001L)

//
// MessageId: IM_EVENT_MINIPORT_REGISTER_FAILED
//
// MessageText:
//
//  %2 failed to register as a Intermediate Miniport.
//
#define IM_EVENT_MINIPORT_REGISTER_FAILED ((NTSTATUS)0xC0080002L)

//
// MessageId: IM_EVENT_BAD_REGISTRY_DATA
//
// MessageText:
//
//  %2 has a problem with this Registry entry: %3.
//
#define IM_EVENT_BAD_REGISTRY_DATA       ((NTSTATUS)0xC0080003L)

#endif /* _IMLOG_ */
