/*++

Copyright (c) 1996  Microsoft Corporation

Module Name:

    ndisreq.h

Abstract:

    defines for NdisRequest related routines

Author:

    Jim Mateer 4-1-97

Revision History:

--*/

#ifndef _NDISREQ_
#define _NDISREQ_


/*++

Routine Description:

    handler for QueryInformation NdisRequests. We note the type of request and
    pass the request on to a common rebundling routine

Arguments:

    See the DDK...

Return Values:

    return the value returned to us by the underlying adapter

--*/

NDIS_STATUS
MPQueryInformation(
	IN	NDIS_HANDLE				MiniportAdapterContext,
	IN	NDIS_OID				Oid,
	IN	PVOID					InformationBuffer,
	IN	ULONG					InformationBufferLength,
	OUT PULONG					BytesWritten,
	OUT PULONG					BytesNeeded
	);

/*++

Routine Description:

    handler for SetInformation NdisRequests. We note the type of request and
    pass the request on to a common rebundling routine

Arguments:

    See the DDK...

Return Values:

    return the value returned to us by the underlying adapter

--*/

NDIS_STATUS
MPSetInformation(
	IN	NDIS_HANDLE				MiniportAdapterContext,
	IN	NDIS_OID				Oid,
	IN	PVOID					InformationBuffer,
	IN	ULONG					InformationBufferLength,
	OUT PULONG					BytesRead,
	OUT PULONG					BytesNeeded
	);

/*++

Routine Description:

    common handler for set and query information routines. An NDIS_REQUEST is
    built and issued to the underlying MP

Arguments:

    See the DDK...

    RequestType also includes NdisRequestLocal which is used to indicate a
    local request meaning the request is originated by the Intermediate driver
    and needs no further completion

Return Values:

    return the value returned to us by the underlying adapter

--*/

NDIS_STATUS
MPNdisRequest(
	IN	PADAPTER				Adapter,
    IN  NDIS_REQUEST_TYPE       RequestType,
	IN	NDIS_OID				Oid,
	IN	PVOID					InformationBuffer,
	IN	ULONG					InformationBufferLength,
	OUT PULONG					BytesReadOrWritten,
	OUT PULONG					BytesNeeded,
    LOCAL_NDISREQUEST_COMPLETION_FUNCTION CompletionFunc
	);

/*++

Routine Description:

    Completion routine for NdisRequest. Stuff our block back on the lookaside
    list and call the appropriate completion routine

Arguments:

    See the DDK...

Return Values:

    None

--*/

VOID
CLRequestComplete(
    IN  NDIS_HANDLE     ProtocolBindingContext,
    IN  PNDIS_REQUEST   NdisRequestBuf,
    IN  NDIS_STATUS     Status
    );

NDIS_STATUS
MakeLocalNdisRequest(
    PADAPTER Adapter,
    NDIS_OID Oid,
    PVOID Buffer,
    ULONG BufferSize,
    LOCAL_NDISREQUEST_COMPLETION_FUNCTION CompletionFunc OPTIONAL
    );

NDIS_STATUS
MPCoRequest(
    IN  NDIS_HANDLE             MiniportAdapterContext,
    IN  NDIS_HANDLE             MiniportVcContext   OPTIONAL,
    IN OUT PNDIS_REQUEST        NdisRequestBuf
    );

/* End Prototypes */

#endif /* _NDISREQ_ */

/* end ndisreq.h */
